/**************************************************************************************

        MUSTUXLIB - THE COMMON LIBRARY FOR ALL MUSTUX APPLICATIONS
        AUTHOR : See AUTHORS file for details

        This software is distributed under the terms of the GNU General Public License
        as specified in the COPYING file.

***************************************************************************************/


#include "MustuxGlobalProperties.hh"

#include <vector>

#include "MustuxDebugger.hh"


// WARNING: MAX_PROPERTIES MUST BE A PRIME NUMBER !!!
// grep one big enough at http://doc.trolltech.com/3.1/primes.html
// if you need to increase it
#define MAX_PROPERTIES 101


class Callback
        {
	public:
	        Callback(void *pObject, void (*pFunction)(QString, QString, void *))
	        {
		        object = pObject;
			function = pFunction;
		};
	        ~Callback() {};

	        void *object;
	        void (*function)(QString, QString, void *);
        };


class KeyData
        {
	public:
	        KeyData() {};
	        KeyData(QString pValue) { value = pValue; };
	        ~KeyData() {};

	        QString value;
	        std::vector<Callback *> callbacks;
        };


namespace MustuxGlobalProperties
        {
	QString  rcFileName;
	QDict<KeyData> dict(MAX_PROPERTIES, FALSE); // not case sensitive
	void MustuxGlobalProperties::update(QString key);
        }


void MustuxGlobalProperties::init(QString fileName)
        {
        PENTER;

	dict.setAutoDelete(false);  // data is deleted when removed
	dict.clear();

        QString userHomePath = QString(getenv("HOME")) + "/";
        rcFileName = userHomePath + fileName;

	QFileInfo rcFileInfo(rcFileName);
	    
	if (rcFileInfo.exists())
	  {
	    QFile rcFile(rcFileName);
	    rcFile.open(IO_ReadOnly);
	    QTextStream rcStream(&rcFile);
	    QString line;
	    
	    while (!(line = rcStream.readLine()).isNull())
	      {
		int pos = line.find('=');
		if (pos == -1)
		  continue;
		
		set(line.left(pos), line.mid(pos + 1));
	      }
	    rcFile.close();
	  }
	else
	  {
	    set("HZOOM_LEVEL", 12);
	    set("DEFAULT_NUM_TRACKS", 6);
	    set("MINIMUM_MULTITRACKAREA_WIDTH", 740);
	    set("MINIMUM_MULTITRACKAREA_HEIGHT", 400);
	    set("SPECTROGRAM_WIDTH", 200);
	    set("SPECTROGRAM_HEIGHT", 100);
	    set("JMB_CLEAR_TIME", 2000);
	    set("JMB_HOLD_SENSITIVENESS_FACTOR", 10);
	    set("JMB_DOUBLE_PRESS_INTERVAL", 200);
	    set("SLICE_WIDTH_FACTOR", 1);
	    set("MP3_DECODE_COMMAND", "mpg123 -w %if");
	    set("OGG_DECODE_COMMAND", "ogg123 -d wav %if");
	    set("MP3_ENCODE_COMMAND", "notlame -b320 %if");
	    set("OGG_ENCODE_COMMAND", "oggenc -b320 %if");
	    set("DEFAULT_SAMPLE_RATE", 44100);
	    set("DEFAULT_BIT_DEPTH", 16);
	  }

        PEXIT;
        }


void MustuxGlobalProperties::save()
        {
	PMESG("Saving MustuxGlobalProperties to %s ... " , (const char*) rcFileName.latin1());

	QFile rcFile(rcFileName);
	if (!rcFile.open(IO_WriteOnly))
	        {
		PERROR("Could not save .protuxrc");
		return;
		}
	QTextStream rcStream(&rcFile);

	QDictIterator<KeyData> it(dict);

	while (it.current())
	        {
		rcStream << it.currentKey().latin1() << "=" << it.current()->value.latin1() << std::endl;
		++it;
		}

	rcFile.close();
	}


void MustuxGlobalProperties::finish()
        {
        PENTER;
        PEXIT;
        }


// deprecated, use set instead (if the key is present, is sets it, otherwise it inserts it)
void MustuxGlobalProperties::insert(QString key, QString value)
        {
	set(key, value);
	}


void MustuxGlobalProperties::remove(QString key)
        {
	// as autoDelete is activated, the value (KeyData *) is automatically deleted
	dict.remove(key);
	}


QString MustuxGlobalProperties::get(QString key)
        {
	KeyData *data = dict.find(key);

	if (data)
	        return data->value;
	return NULL;
	}


int MustuxGlobalProperties::get_int(QString key)
        {
	KeyData *data = dict.find(key);

	if (data)
	        {
		bool isOk;
		int i = data->value.toInt(&isOk);
		if (isOk)
		        return i;
		}
	// if key does not exist or is not an integer, return 0
	// Warning: if it returns 0, it does not automatically mean
	// that the key didn't exist or it wasn't an int... 0 is a valid int !
	return 0;
	}


float MustuxGlobalProperties::get_float(QString key)
        {
	KeyData *data = dict.find(key);

	if (data)
	        {
		bool isOk;
		float f = data->value.toFloat(&isOk);
		if (isOk)
		        return f;
		}
	// if key does not exist or is not an float, return 0
	// Warning: if it returns 0, it does not automatically mean
	// that the key didn't exist or it wasn't a float... 0 is a valid float !
	return 0;
}


void MustuxGlobalProperties::set(QString key, QString value)
        {
	if (KeyData *data = dict.find(key))
	        {
		data->value = value;
		}
	else
	        {
		KeyData *data = new KeyData(value);
		dict.insert(key, data);
		}
	update(key);
	}


void MustuxGlobalProperties::set(QString key, int value)
        {
	if (KeyData *data = dict.find(key))
	        {
		data->value = QString::number(value);
		}
	else
	        {
		KeyData *data = new KeyData(QString::number(value));
		dict.insert(key, data);
		}
	update(key);
	}


void MustuxGlobalProperties::set(QString key, float value)
        {
	if (KeyData *data = dict.find(key))
	        {
		data->value = QString::number(value);
		}
	else
	        {
		  KeyData *data = new KeyData(QString::number(value));
		  dict.insert(key, data);
		}
	update(key);
	}


void MustuxGlobalProperties::show_dict()
        {
	PMESG("Showing dict...");
	QDictIterator<KeyData> it(dict);

	while (it.current())
	        {
		PMESG("         Property %s=%s", it.currentKey().latin1(), it.current()->value.latin1());
		++it;
		}
	}


bool MustuxGlobalProperties::exists(QString key)
        {
	if (dict.find(key))
	        return true;
	return false;
	}


// Usage for MustuxGlobalProperties::register_callback_function():
// - first argument: the key for which the callback function will be registered
// - second argument: the callback function to be registered. This function will be
//   called whenever the value of the key changes.
//   (the callback function must be a static function, and must have following prototype:
//    void functionName(QString key, QString value, void *object)
//    where object is a pointer to the object asking for a function to be registered)
// - third argument: a pointer to the object asking for a function to be registered, this is usually 'this'
//   (this pointer will then be given as third argument to the callback function)
//
// Example: class myClass wants to be notified when key "myKey" changes, it's callback function is myClass::my_callback_function,
// so myClass does the following (usually in the constructor):
// MustuxGlobalProperties::register_callback_function("mykey", my_callback_function, this)
// and now if any other class changes "myKey", my_callback_function gets called.

void MustuxGlobalProperties::register_callback_function(QString key, void (*callbackFunction)(QString, QString, void *), void *object)
        {
	if (KeyData *data = dict.find(key))
	        {
		Callback *callback = new Callback(object, callbackFunction);
		data->callbacks.push_back(callback);
		}
	}


// this function is used only internaly
void MustuxGlobalProperties::update(QString key)
        {
	if(KeyData *data = dict.find(key))
	        {
		std::vector<Callback *>::iterator it = data->callbacks.begin();
		std::vector<Callback *>::iterator it_e = data->callbacks.end();
		while(it != it_e)
		        {
			void (*callbackFunction)(QString, QString, void *);
			callbackFunction = (*it)->function;
			callbackFunction(key, data->value, (*it)->object);
			it++;
			}
		}
	}

//eof

